indexing

   description: "Class that covers the POSIX file system code."

   author: "Berend de Boer"
   date: "$Date: Fri, 13 Oct 2000 13:36:48 +0200 $";
   revision: "$Revision: 1.5 $"

class 

   POSIX_FILE_SYSTEM

inherit

   POSIX_BASE

   STDC_FILE_SYSTEM
      redefine
         is_modifiable,
         is_readable
      end

   PAPI_UNISTD

   PAPI_STAT

   PAPI_UTIME
   
   
creation
   
   make


feature -- directory access
   
   change_directory, chdir (a_directory: STRING) is
         -- Changes the current working directory
      local
         r: INTEGER
      do
         portable_path.make_expand (a_directory)
         r := posix_chdir (sh.string_to_pointer (portable_path))
         if r = -1 then
            raise_posix_error
         end
      end

   current_directory, getcwd, pwd: STRING is
         -- The current directory
      local
         buf: STDC_DYNAMIC_MEMORY
         r: POINTER
         buf_size: INTEGER
      do
         buf_size := 1024
         create buf.allocate (buf_size)
         r := posix_getcwd (buf.ptr, buf_size)
         if r = default_pointer then
            raise_posix_error
         end
         Result := sh.pointer_to_string (buf.ptr)
      end

   make_directory, mkdir (a_directory: STRING) is
         -- Makes a directory, only accessible by owner
      local
         r: INTEGER
      do
         portable_path.make_expand (a_directory)
         r := posix_mkdir (sh.string_to_pointer (portable_path), S_IRUSR + S_IWUSR + S_IXUSR)
         if r = -1 then
            raise_posix_error
         end
      end

   remove_directory, rmdir (a_directory: STRING) is
         -- Removes a directory
      do
         portable_path.make_expand (a_directory)
         safe_call (posix_rmdir (sh.string_to_pointer (portable_path)))
      end


feature -- read/write permissions
   
   chmod, change_mode (a_path: STRING; a_mode: INTEGER) is
         -- Changes file mode
      require
         valid_path: a_path /= Void and then not a_path.is_empty
      local
         r: INTEGER
      do
         portable_path.make_expand (a_path)
         r := posix_chmod (sh.string_to_pointer (portable_path), a_mode)
         if r = -1 then
            raise_posix_error
         end
      end

   permissions (a_path: STRING): POSIX_PERMISSIONS is
         -- return the permissions object (a new one every time!) for
         -- the given file
      require
         valid_path: a_path /= Void and then not a_path.is_empty
      do
         Result := status (a_path).permissions
      end
   
   set_read_only (a_path: STRING) is
         -- Make given file read_only
      require
         valid_path: a_path /= Void and then not a_path.is_empty
      local
         perm: POSIX_PERMISSIONS
      do
         perm := status (a_path).permissions
         perm.set_allow_write (False)
         perm.apply
      end


feature -- file statistics

   status (a_path: STRING): POSIX_STATUS is
         -- Gets information about a file
      require
         valid_path: a_path /= Void and then not a_path.is_empty
      do
         create {POSIX_STATUS_PATH} Result.make (a_path)
      end

   touch (a_path: STRING) is
      -- Sets the modification and access times of `a_path' to the 
      -- current time of day. 
      local
         r: INTEGER
      do
         portable_path.make_expand (a_path)
         r := posix_utime (sh.string_to_pointer (portable_path), -1, -1)
         if r = -1 then
            raise_posix_error
         end
      end

   utime (a_path: STRING; access_time, modification_time: POSIX_TIME) is
         -- Sets file access and modification times
      local
         r: INTEGER
      do
         portable_path.make_expand (a_path)
         r := posix_utime (sh.string_to_pointer (portable_path), access_time.value, modification_time.value)
         if r = -1 then
            raise_posix_error
         end
      end


feature -- accessibility of files

   last_access_result: INTEGER
         -- value of last access test

   is_accessible, access (a_path: STRING; a_mode: INTEGER): BOOLEAN is
         -- Tests for file accessibility
      do
         portable_path.make_expand (a_path)
         last_access_result := posix_access (sh.string_to_pointer (portable_path), a_mode)
         Result := last_access_result = 0
      end

   is_existing (a_path: STRING): BOOLEAN is
         -- tests if file does exist, not if it is readable or writable by 
         -- this program!
         -- uses real user ID and real group ID instead of effective ones
      do
         Result := is_accessible (a_path, F_OK)
      end

   is_empty (a_path: STRING): BOOLEAN is
         -- True if file exists and has a size equal to zero.
      require
         exists: is_existing (a_path)
      do         
         Result := status (a_path).size = 0
      end

   is_executable (a_path: STRING): BOOLEAN is
         -- tests if file is executable by this program
      do
         Result := is_accessible (a_path, X_OK)
      end
   
   is_modifiable (a_path: STRING): BOOLEAN is
         -- tests if file is readable and writable by this program
         -- uses real user ID and real group ID instead of effective ones
      do
         Result := is_accessible (a_path, R_OK + W_OK)
      end
   
   is_readable (a_path: STRING): BOOLEAN is
         -- tests if file is readable by this program
         -- uses real user ID and real group ID instead of effective ones
      do
         Result := is_accessible (a_path, R_OK)
      end

   is_writable (a_path: STRING): BOOLEAN is
         -- tests if file is writable by this program
         -- uses real user ID and real group ID instead of effective ones
      do
         Result := is_accessible (a_path, W_OK)
      end


feature -- further directory access
   
   link (existing, new: STRING) is
         -- Creates a hard link to a file
      require
         different_names: not existing.is_equal(new)
      local
         cexisting,
         cnew: POINTER
         expanded_new: POSIX_PATH
         r: INTEGER
      do
         portable_path.make_expand (existing)
         cexisting := sh.string_to_pointer (portable_path)
         create expanded_new.make (new)
         cnew := sh.string_to_pointer (expanded_new)
         r := posix_link (cexisting, cnew)
         if r = -1 then
            raise_posix_error
         end
      end
   
   unlink (a_path: STRING) is
         -- Removes a directory entry (equal to remove)
         -- it's not an error if path does not exist
      require
         valid_path: a_path /= Void and then not a_path.is_empty
      local
         r: INTEGER
      do
         portable_path.make_expand (a_path)
         r := posix_unlink (sh.string_to_pointer (portable_path))
      end


feature -- directory browsing

   browse_directory (a_directory: STRING): POSIX_DIRECTORY is
      require
         valid_dir: a_directory /= Void and then not a_directory.is_empty
      do
         create Result.make (a_directory)
      end
   

feature -- mkfifo

   make_fifo, mkfifo (a_path: STRING; a_mode: INTEGER) is
         -- Makes a FIFO special file
      require
         valid_path: a_path /= Void and then not a_path.is_empty
      do
         safe_call (posix_mkfifo (sh.string_to_pointer (a_path), a_mode))
      end
   
   
feature {NONE} -- c interface



end -- class POSIX_FILE_SYSTEM
